import os
from itertools import product, chain
import json
import argparse
import glob
import re

import numpy as np
from scipy import stats
import tqdm

if __name__=="__main__":
  sources = chain(
	product(
          ["heavytail_low", "heavytail_medium", "heavytail_high", "heavytail_veryhigh", "heavytail_veryveryhigh"],
          ["poisson"],
          [9]),
	product(
          ["heavytail_low"],
          ["bursty_low", "bursty_medium", "bursty_high", "bursty_veryhigh", "bursty_veryveryhigh"],
          [9]),
	product(
          ["regular"],
          ["poisson", "bursty_low", "bursty_medium", "bursty_high", "bursty_veryhigh", "bursty_veryveryhigh"],
          [9]),
	product(
          ["erdos"],
          ["poisson", "bursty_low", "bursty_medium", "bursty_high", "bursty_veryhigh", "bursty_veryveryhigh"],
          [8]),
        )

  parser = argparse.ArgumentParser(
      description='aggreagates separate results into fewer, bigger files')
  parser.add_argument('input_prefix', type=str)
  parser.add_argument('output_prefix', type=str)

  args = parser.parse_args()

  for static_model, temporal_model, deg_id in sources:
    dist_files = glob.glob(os.path.join(
                          args.input_prefix, static_model,
                          temporal_model, "*", str(deg_id),
                          f"dist.jsonl"))
    dist = {}
    for filename in tqdm.tqdm(list(dist_files)):
      with open(filename) as f:
        pattern = os.path.join(args.input_prefix, static_model,
                                 temporal_model, r"(\d+)",
                                 str(deg_id), r".*\.jsonl")
        m = re.fullmatch(pattern, filename)
        size = int(m[1])  # if this fails then the pattern is wrong :/
        for line in f:
          try:
            j = json.loads(line)
            dt = j["dt"]
            seed = j["seed"]
            prob_dist = j["prob_dist_type"]

            if size not in dist:
              dist[size] = {}

            if prob_dist not in dist[size]:
              dist[size][prob_dist] = {}

            if dt not in dist[size][prob_dist]:
              dist[size][prob_dist][dt] = {(i, j): 0 for i in range(3) for j in range(3)}

            for c in j["counts"]:
              dist[size][prob_dist][dt][(c["in"], c["out"])] += c["count"]
          except json.JSONDecodeError:
            continue

    for size in dist:
      for prob_dist in dist[size]:
        for dt in dist[size][prob_dist]:
          counts = []
          for (i, j), c in dist[size][prob_dist][dt].items():
            counts.append({"in": i, "out": j, "count": c})
          dist[size][prob_dist][dt] = {"counts": counts, "size": size, "dt": dt}

    output_path = os.path.join(args.output_prefix,
        f"{static_model}-{temporal_model}-{deg_id}-dist.json")
    print(output_path)
    with open(output_path, 'w') as f:
      json.dump(dist, f)
