import glob
import csv
import sqlite3
import argparse
import os

import tqdm

from network_configs import StaticModel, TemporalModel, ProbDist 

if __name__ == "__main__":
  parser = argparse.ArgumentParser(description="Fits critical exponents and dtc using grid search")
  parser.add_argument('static', type=StaticModel)
  parser.add_argument('temporal', type=TemporalModel)

  args = parser.parse_args()

  filenames = glob.glob(f"networks/random_large_networks/param-fit/{args.static.value}-{args.temporal.value}-*.csv")

  dbname = f"networks/random_large_networks/param-fit/{args.static.value}-{args.temporal.value}.db"

  # "truncate" the database
  if os.path.isfile(dbname):
    os.remove(dbname)
  
  con = sqlite3.connect(dbname)
  cur = con.cursor()

  cur.execute('''CREATE TABLE params
               (beta DECIMAL(3,2),
                beta_prime DECIMAL(3,2),
                nu_para DECIMAL(3,2),
                nu_perp DECIMAL(3,2),
                dtc DECIMAL(8,7),
                rmsd_p real,
                rmsd_rho real,
		rmsd_mass real,
		rmsd_volume real)''')


  cur.execute("CREATE INDEX params_dtc ON params (dtc)")

  for filename in tqdm.tqdm(filenames):
    with open(filename) as f:
      reader = csv.DictReader(f)
      for row in reader:
        if row["rmsd_p"] != 'inf' and row["rmsd_rho"] != 'inf':
          keys = list(row.keys())
          cur.execute("INSERT INTO params (" + ', '.join(keys) + ") VALUES (" + ', '.join([row[k] if row[k] != 'inf' else "'+Infinity'" for k in keys]) + ")")
    con.commit()

  con.close()
