import sqlite3
import json
import sys
import csv

import numpy as np
from scipy import stats

from network_configs import StaticModel, TemporalModel, ProbDist

def total_error(row):
  return row["rmsd_p"] + row["rmsd_rho"] + row["rmsd_mass"] + row["rmsd_volume"]

if __name__=="__main__":
  exponents = ["beta", "beta_prime", "nu_para", "nu_perp"]
  rmsds = ["rmsd_rho", "rmsd_p", "rmsd_mass", "rmsd_volume"]

  beta =    {1: 0.276, 2: 0.583, 3: 0.813, 4: 1.0}
  nu_perp = {1: 1.096, 2: 0.733, 3: 0.584, 4: 0.5}
  nu_para = {1: 1.734, 2: 1.295, 3: 1.11,  4: 1.0}
  beta_p = beta

  print('static', 'temporal', 'deg', 'dims', 'dtc', *[e+"_error" for e in exponents], "alpha", "delta")
  with open('network-configs.csv', newline='') as csvfile:
    reader = csv.DictReader(csvfile)
    for args in reader:
      dbfilename = f"networks/random_large_networks/param-fit/{args['static']}-{args['temporal']}.db"
      con = sqlite3.connect(dbfilename)
      cur = con.cursor()

      dims = int(args['dims'])

      params = {
        "beta": beta[dims],
        "beta_prime": beta_p[dims],
        "nu_para": nu_para[dims],
        "nu_perp": nu_perp[dims]}

      dtc_error = np.inf
      dtc = None
      query = "SELECT " + ",".join([k + " AS " + k for k in exponents + rmsds + ['dtc']]) + " FROM params WHERE " + \
        ' AND '.join([k + "=" + "%.2f"%params[k] for k in exponents])
      print(query, file=sys.stderr)
      for row in cur.execute(query):
        row_dict = dict(zip(exponents + rmsds + ['dtc'], row))
        if total_error(row_dict) < dtc_error:
          dtc_error = total_error(row_dict)
          dtc = row_dict['dtc']

      error_bars = {}
      for param in exponents:
        comb = set(exponents) - set([param])
        query = "SELECT " + ", ".join([k + " AS " + k for k in exponents + rmsds + ['dtc']]) + " FROM params WHERE " + \
          ' AND '.join([k + "=" + "%.2f"%params[k] for k in comb]) + f" AND dtc = {dtc} ORDER BY {param}"
        print(query, file=sys.stderr)
        min_v = np.inf
        max_v = -np.inf
        for row in cur.execute(query):
          row_dict = dict(zip(exponents + rmsds + ['dtc'], row))
          if total_error(row_dict) <= dtc_error:
            min_v = min(min_v, row_dict[param])
            max_v = max(max_v, row_dict[param])
        error_bars[param] = max_v - min_v

      prob_dist = ProbDist.deterministic
      deg = int(args['deg'])
      single_filename = f"networks/aggregated_networks/{args['static']}-{args['temporal']}-{deg}-single.json"
      homogeneous_filename = f"networks/aggregated_networks/{args['static']}-{args['temporal']}-{deg}-homogeneous.json"
      with open(single_filename) as sngl_f, open(homogeneous_filename) as homo_f:
        single_summaries = json.load(sngl_f)
        homogeneous_summaries = json.load(homo_f)

        max_size_s = max(single_summaries, key=int)
        size = int(max_size_s)
        s = single_summaries[max_size_s][prob_dist.value]
        time = np.array(s[str(dtc)]["time-bin-edges"][0][:-1])
        mk_survival = np.array(s[str(dtc)]["km-survival-dist"][0])
        max_t = 0.04*size**(nu_para[dims]/(nu_perp[dims]*dims))
        min_t = 2e0
        mask = (time > min_t) & (time < max_t)
        res = stats.linregress(np.log(time[mask]), np.log(mk_survival[mask]))
        delta = -res.slope

        max_size_s = max(homogeneous_summaries, key=int)
        size = int(max_size_s)
        s = homogeneous_summaries[max_size_s][prob_dist.value]
        time = np.array(s[str(dtc)]["time-bin-edges"][0][:-1])
        density = np.array(s[str(dtc)]["density"][0])
        max_t = 0.04*size**(nu_para[dims]/(nu_perp[dims]*dims))
        min_t = 2e0
        mask = (time > min_t) & (time < max_t)
        res = stats.linregress(np.log(time[mask]), np.log(density[mask]))
        alpha = -res.slope

      print(args['static'], args['temporal'], deg, dims, dtc, *["%.2f"%error_bars[e] for e in exponents], alpha, delta)
