import csv

if __name__=="__main__":
  items = {}
  with open("optimum-dts.csv", newline='') as f:
    reader = csv.DictReader(f, delimiter=' ')
    for row in reader:
      static = row['static']
      temporal = row['temporal']
      dtc = row['dtc']
      alpha = row['alpha']
      delta = row['delta']

      beta_error = row['beta_error']
      beta_prime_error = row['beta_prime_error']
      nu_para_error = row['nu_para_error']
      nu_perp_error = row['nu_perp_error']

      temporal_type = temporal
      if temporal.startswith("bursty"):
        temporal_type = "bursty"
      elif temporal.startswith("self-exciting"):
        temporal_type = "self-exciting" 
      if static not in items:
        items[static] = {}
      if temporal_type not in items[static]:
        items[static][temporal_type] = []

      items[static][temporal_type].append((
        static, temporal, dtc, beta_error, beta_prime_error, nu_para_error, nu_perp_error, float(alpha), float(delta)))

  for static in items:
    static_string = static.capitalize()
    if static == "grid_1":
      static_string = "1D lattice"
    elif static == "grid_2":
      static_string = "2D square lattice"
    elif static == "grid_3":
      static_string = "3D square lattice"
    elif static == "grid_4":
      static_string = "4D square lattice"
    elif static == "erdos":
      static_string = "Erdős--Rényi $\\langle k \\rangle = 8$"
    elif static == "regular":
      static_string = "Random 9-regular"

    print(f"{static_string}\\\\")

    for temporal_type in items[static]:
      temporal_type_string = temporal_type.capitalize()

      if temporal_type == "self-exciting":
        temporal_type_string = "Hawkes self-exciting"

      print(f"\\hspace{{1em}}{temporal_type_string}", end="")

      if temporal_type != "poisson":
        print("\\\\")

      for s, t, dtc, beta_error, beta_prime_error, nu_para_error, nu_perp_error, alpha, delta in items[static][temporal_type]:
        splitted = t.split('_')

        params_string = ""
        if temporal_type == "self-exciting":
          params_string = f"$\\mu={splitted[1]}\\ \\alpha={splitted[2]}\\ \\theta={splitted[3]}$"
        elif temporal_type == "bursty":
          params_string = f"$\\gamma={splitted[1]}$"

        if float(beta_error) < 0.01:
          beta_error = "0.01"
        if float(beta_prime_error) < 0.01:
          beta_prime_error = "0.01"
        if float(nu_para_error) < 0.01:
          nu_para_error = "0.01"
        if float(nu_perp_error) < 0.01:
          nu_perp_error = "0.01"

        if temporal_type != "poisson":
          print("\\hspace{2em}", end="")

        print(f"{params_string} & {dtc} & {beta_error} & {beta_prime_error} & {nu_para_error} & {nu_perp_error} & {alpha:.4f} & {delta:.4f}\\\\")
