import json
import os

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.dates as mdates
from matplotlib.ticker import FuncFormatter
import tqdm

import matplotlib 
matplotlib.rc('legend', fontsize='x-small', handlelength=1, markerscale=2, columnspacing=0.8, labelspacing=0.3, borderpad=0.4)
matplotlib.rc('lines', linewidth=0.75, markersize=2)
matplotlib.rc('axes', labelpad=2)
matplotlib.rc('axes.spines', top=False, right=False)
matplotlib.rc('xtick.major', pad=2)
matplotlib.rc('xtick.minor', pad=2)
matplotlib.rc('ytick.major', pad=2)
matplotlib.rc('ytick.minor', pad=2)

if __name__=="__main__":
  # networks = ["air", "transport", "mobile", "twitter", "random_large_networks/regular/poisson/1024/9"]
  network = "random_large_networks/regular/poisson/1024/9"
  dts = [0.07, 0.092]
  letters = ['g', 'h']

  fig, axes = plt.subplots(1, 2, sharex=True,
	          figsize=(3.375*1.5, 1.4*1.5),
                  gridspec_kw={'wspace': 0.5})
  axes = axes.flatten()

  filename = os.path.join("networks", network, "out-comp-sizes.jsonl")
  if os.path.exists(filename):
    with open(filename) as f:
      for ax, dt, letter in zip(axes, dts, letters):
        f.seek(0)
        for l in f:
          j = json.loads(l)
          if np.isclose(j['dt'], dt):
            # mass_aspect = abs((mdates.epoch2num(j["time-bin-edges"][-1]) - mdates.epoch2num(j["time-bin-edges"][0]))/\
            #                      (j["mass-bin-edges"][-1] - j["mass-bin-edges"][0]))/((len(j["time-bin-edges"])-1)/\
            #                      (len(j["mass-bin-edges"])-1))
            # mass_ax.imshow(np.transpose(j["mass-bin-counts"]) > 0, aspect=mass_aspect,
            #                  origin="lower", cmap="binary", vmin=0, vmax=1, interpolation='none',
            #                  extent=[mdates.epoch2num(j["time-bin-edges"][0]), mdates.epoch2num(j["time-bin-edges"][-1]),
            #                            j["mass-bin-edges"][0], j["mass-bin-edges"][-1]])
            # mass_ax.set_ylabel("Component Mass\n(# of events)")
            # mass_ax.ticklabel_format(axis='y', style='sci', useMathText=True, scilimits=(0,0))

            volume_aspect = abs((mdates.epoch2num(j["time-bin-edges"][-1]) - mdates.epoch2num(j["time-bin-edges"][0]))/\
                                 (j["volume-bin-edges"][-1] - j["volume-bin-edges"][0]))/((len(j["time-bin-edges"])-1)/\
                                 (len(j["volume-bin-edges"])-1))
            ax.imshow(np.transpose(j["volume-bin-counts"]) > 0, aspect=volume_aspect,
                             origin="lower", cmap="binary", vmin=0, vmax=1, interpolation='none',
                             extent=[mdates.epoch2num(j["time-bin-edges"][0]), mdates.epoch2num(j["time-bin-edges"][-1]),
                                       j["volume-bin-edges"][0], j["volume-bin-edges"][-1]])
            ax.set_ylabel("Component Volume\n(# of nodes)")
            ax.ticklabel_format(axis='y', style='sci', useMathText=True, scilimits=(0,0))

            # lifetime_aspect = abs((mdates.epoch2num(j["time-bin-edges"][-1]) - mdates.epoch2num(j["time-bin-edges"][0]))/\
            #                      (mdates.epoch2num(j["lifetime-bin-edges"][-1]) - mdates.epoch2num(j["lifetime-bin-edges"][0])))/\
            #                      ((len(j["time-bin-edges"])-1)/(len(j["lifetime-bin-edges"])-1))
            # lifetime_ax.imshow(np.transpose(j["lifetime-bin-counts"]) > 0, aspect=lifetime_aspect,
            #                  origin="lower", cmap="binary", vmin=0, vmax=1, interpolation='none',
            #                  extent=[mdates.epoch2num(j["time-bin-edges"][0]), mdates.epoch2num(j["time-bin-edges"][-1]),
            #                            mdates.epoch2num(j["lifetime-bin-edges"][0]), mdates.epoch2num(j["lifetime-bin-edges"][-1])])
            # lifetime_locator = mdates.AutoDateLocator(minticks=2, maxticks=4)
            # lifetime_formatter = FuncFormatter(lambda t, _: int(t - mdates.epoch2num(j["lifetime-bin-edges"][0])))
            # if network == "transport":
            #   lifetime_formatter = FuncFormatter(lambda t, _: int((t - mdates.epoch2num(j["lifetime-bin-edges"][0]))*24 + 0.5))
            # elif network.startswith("random_large_networks"):
            #   lifetime_formatter = FuncFormatter(lambda t, _: int((t - mdates.epoch2num(j["lifetime-bin-edges"][0]))*24*3600 + 0.5))
            #   lifetime_ax.ticklabel_format(axis='y', style='sci', useMathText=True, scilimits=(0,0))
            # lifetime_ax.yaxis.set_major_locator(lifetime_locator)
            # lifetime_ax.yaxis.set_major_formatter(lifetime_formatter)

            # lifetime_unit = "\n(days)"
            # if network == "transport":
            #   lifetime_unit = "\n(hours)"
            # elif network.startswith("random_large_networks"):
            #   lifetime_unit = ""
            # lifetime_ax.set_ylabel("Component Lifetime" +  lifetime_unit)

            ax.spines["right"].set_visible(False)
            ax.spines["top"].set_visible(False)
            ax.spines["left"].set_visible(False)
            ax.spines["bottom"].set_visible(False)

            locator = mdates.AutoDateLocator(minticks=3, maxticks=5)

            formatter = FuncFormatter(lambda t, _: int(t - mdates.epoch2num(j["time-bin-edges"][0])))
            if network == "transport":
              formatter = FuncFormatter(lambda t, _: int((t - mdates.epoch2num(j["time-bin-edges"][0]))*24 + 0.5))
            elif network.startswith("random_large_networks/regular/"):
              formatter = FuncFormatter(lambda t, _: int((t - mdates.epoch2num(j["time-bin-edges"][0]))*24*3600 + 0.5))
            ax.xaxis.set_major_locator(locator)
            ax.xaxis.set_major_formatter(formatter)

            time_unit = "(days)"
            if network == "transport":
              time_unit = "(hours)"
            elif network.startswith("random_large_networks/regular/"):
              time_unit = ""
            ax.set_xlabel("Event time " + time_unit)

            ax.text(-0.22, 1.35, f"({letter})", transform=ax.transAxes)

  fig.savefig(os.path.join("figures", "dists", network, f"out-comps.pdf"),
                      bbox='tight', pad_inches=0.1)
  plt.close(fig)
