import json
import string
import itertools

import matplotlib.pyplot as plt

from network_configs import StaticModel, TemporalModel, ProbDist
from plot_scaling import plot_system

import matplotlib 
matplotlib.rc('legend', fontsize='x-small', handlelength=1, markerscale=2, labelspacing=0.3, columnspacing=0.7)
matplotlib.rc('lines', linewidth=0.75, markersize=2)
matplotlib.rc('axes', labelpad=2)
matplotlib.rc('axes.spines', top=False, right=False)
matplotlib.rc('xtick.major', pad=2)
matplotlib.rc('xtick.minor', pad=2)
matplotlib.rc('ytick.major', pad=2)
matplotlib.rc('ytick.minor', pad=2)

if __name__=="__main__":
  beta =    {1: 0.276, 2: 0.583, 3: 0.814, 4: 1.0}
  nu_para = {1: 1.734, 2: 1.295, 3: 1.110, 4: 1.0}
  nu_perp = {1: 1.097, 2: 0.733, 3: 0.584, 4: 0.5}
  beta_p = beta


  fig = plt.figure(constrained_layout=True, figsize=(3.375*2*1.5, 2.00*2*1.5))
  subfigs = fig.subfigures(1, 2, wspace=0.1, hspace=0.15)
  left_axes = subfigs[0].subplots(4, 2, sharex=True, sharey='row',
                                   gridspec_kw={'wspace': 0.0, 'hspace':0.0, 'height_ratios': [1]*4, 'width_ratios': [1]*2})
  right_axes = subfigs[1].subplots(4, 2, sharex=True, sharey='row',
                                   gridspec_kw={'wspace': 0.0, 'hspace':0.0, 'height_ratios': [1]*4, 'width_ratios': [1]*2})

  axes = list(itertools.chain(*itertools.chain(*zip(left_axes, right_axes))))
  for static, temporal, dtc_s, offset in [(StaticModel.regular, TemporalModel.bursty_280, '0.09046', 0), (StaticModel.erdos, TemporalModel.self_exciting_02_08_10, '0.07932', 1)]:
    prob_dist = ProbDist.deterministic

    deg_id = 9
    dims = 4

    survival_time_lims = (1e-3, 1e0)

    mass_tau_lim = (1e-4, 3e1)
    mass_lim = (1e-3, 5e0)

    volume_tau_lim = (1e-4, 3e1)
    volume_lim = (1e-3, 5e0)

    density_time_lims = (1e-3, 1e0)
    static_density_tau_lims = (5e-2, 5e1)
    susceptibility_tau_lims = (1e-4, 5e1)

    if static is StaticModel.erdos:
      deg_id = 8

    mass_ax, mass_time_ax, \
    volume_ax, volume_time_ax, \
    static_density_ax, density_time_ax,\
    susceptibility_ax, mk_survival_ax = axes[offset::2]

    plot_system(static, temporal, prob_dist, deg_id, dims, dtc_s,
                  beta[dims], beta_p[dims], nu_para[dims], nu_perp[dims],
                  mass_ax, volume_ax, static_density_ax, susceptibility_ax,
                  mk_survival_ax, density_time_ax, mass_time_ax, volume_time_ax)

    # for ax1, ax2 in zip(axes[0::2], axes[1::2]):
    #   ax1.get_shared_y_axes().join(ax1, ax2)
    #   ax2.set_yticklabels([])
    #   ax2.autoscale()
    #   ax2.set_ylabel("")

    # for ax in axes[:-1]:
    #   ax.legend().set_visible(False)

    mk_survival_ax.set_xlim(survival_time_lims)
    mass_ax.set_xlim(mass_tau_lim)
    mass_ax.set_ylim(mass_lim)
    volume_ax.set_xlim(volume_tau_lim)
    volume_ax.set_ylim(volume_lim)

    density_time_ax.set_xlim(density_time_lims)
    susceptibility_ax.set_xlim(susceptibility_tau_lims)
    static_density_ax.set_xlim(static_density_tau_lims)

  for ax in axes[1::2]:
    ax.set_ylabel("")

  for ax in axes[:-4]:
    ax.set_xlabel("")

  axes[-2].legend().set_visible(False)
  axes[-1].legend(ncol=2, loc='lower left')

  for i, ax, letter in zip(itertools.count(), axes, string.ascii_lowercase):
    x = -0.28
    if i%2:
      x = -0.15
    ax.text(x, 0.94, f"({letter})", transform=ax.transAxes)

  fig.savefig(f"figures/scaling-paper.pdf",
      bbox_inches='tight', pad_inches=0.1)
