import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

import matplotlib 
matplotlib.rc('legend', fontsize='x-small', handlelength=1, markerscale=2)
matplotlib.rc('lines', linewidth=0.75, markersize=2)
matplotlib.rc('axes', labelpad=2)
matplotlib.rc('axes.spines', top=False, right=False)
matplotlib.rc('xtick.major', pad=2)
matplotlib.rc('xtick.minor', pad=2)
matplotlib.rc('ytick.major', pad=2)
matplotlib.rc('ytick.minor', pad=2)


if __name__=="__main__":
  timestamps = {
      "example/timestamps/bursty_2.05-timestamp.csv": "Bursty $exponent=2.05$",
      "example/timestamps/bursty_2.2-timestamp.csv": "Bursty $exponent=2.2$",
      "example/timestamps/bursty_5.2-timestamp.csv": "Bursty $exponent=5.2$",
      "example/timestamps/poisson-timestamp.csv": "Poisson $\\lambda=1$",
      "example/timestamps/self-exciting_0.2_0.8_0.5-timestamp.csv": "Hawkes $\\mu=0.2$, $\\alpha=0.8$ and $\\theta=0.5$"}
  
  
  density_fig, density_axes = plt.subplots(2, 3,
    			   figsize=(3.375*1.5, 2.00*1.5),
    			   gridspec_kw={'wspace': 0.3, 'hspace': 0.5})
  density_axes = density_axes.flatten()

  hist_fig, hist_axes = plt.subplots(2, 3, sharex=True,
    			   figsize=(3.375*1.5, 2.00*1.5),
    			   gridspec_kw={'wspace': 0.3, 'hspace': 0.5})
  hist_axes = hist_axes.flatten()

  sample_fig, sample_axes = plt.subplots(5, 1, sharex=True,
    			   figsize=(3.375*1.5, 3.20*1.5),
    			   gridspec_kw={'wspace': 0.3, 'hspace': 2.0})
  sample_axes = sample_axes.flatten()
  
  
  for filename, density_ax, hist_ax, sample_ax in zip(timestamps, density_axes, hist_axes, sample_axes):
    ds = pd.read_csv(filename, squeeze=True)
    title = timestamps[filename]
    ds.plot.density(ax=density_ax, title=title)
    density_ax.set_xlabel("Time")
    density_ax.set_xlabel("Density (Gaussian kernel)")

    bins = np.logspace(0, 2, base=10, num=50)
    ds.diff().plot.hist(ax=hist_ax, bins=bins, density=True, title=title)
    hist_ax.set_xscale("log")
    hist_ax.set_yscale("log")
    hist_ax.set_xlabel("IET")
    hist_ax.set_ylabel("PDF(IET)")

    sample_max_t = 256
    sample = ds[ds <= sample_max_t]
    sample_ax.plot(sample, np.random.rand(len(sample)),
        color='black', marker='.', ls='', ms=1)
    sample_ax.set_title(title)
    sample_ax.spines["left"].set_visible(False)
    if (filename == list(timestamps)[-1]):
      sample_ax.set_xlabel("Time")
    sample_ax.set_ylim(-0.4, +1.4)
    sample_ax.set_xlim(0, sample_max_t)
    sample_ax.set_yticks([])


  
  density_fig.savefig("figures/timeseries-density.pdf",
      bbox_inches='tight', pad_inches=0.1)
  hist_fig.savefig("figures/timeseries-hist.pdf",
      bbox_inches='tight', pad_inches=0.1)
  sample_fig.savefig("figures/timeseries-sample.pdf",
      bbox_inches='tight', pad_inches=0.1)
