import argparse

import numpy as np
import networkx as nx


def plaw(exponent, x_min, size, state):
  p = state.random(size)
  return x_min*(1.0 - p)**(1.0/(1.0 - exponent))

if __name__ == "__main__":
  parser = argparse.ArgumentParser(
      description='generates random regular based temporal network.')
  parser.add_argument('--seed', required=True, type=int)
  parser.add_argument('--nodes', required=True, type=int)
  parser.add_argument('--average-degree', required=True, type=int)
  parser.add_argument('--exponent', required=True, type=float)

  args = parser.parse_args()

  state = np.random.default_rng(args.seed)

  if args.exponent <= 2.0:
    raise "exponent should be larger than 2"

  if args.average_degree <= 0:
    raise "average degree should be positive"

  x_min = args.average_degree*(args.exponent-2.0)/(args.exponent-1.0)
  degree_series = plaw(args.exponent, x_min, args.nodes, state)

  net = nx.expected_degree_graph(degree_series,
      seed=int(state.integers(2**32 - 1)), selfloops=False)

  for i, j in net.edges():
    print(i, j)
