#!/bin/bash
#SBATCH --cpus-per-task 128
#SBATCH --ntasks-per-node 1
#SBATCH --nodes 1
#SBATCH --account=project_2003348
#SBATCH --time=2:00:00
#SBATCH --job-name homogeneous_air
#SBATCH -o /dev/null
#SBATCH -e /dev/null

set -ex


tmpdir=$(mktemp -d --tmpdir=/dev/shm)
prefix=networks/air;

mkdir -p $tmpdir
mkdir -p $prefix

parallel -j $SLURM_CPUS_PER_TASK --trim lr --colsep ',' --header : --shuf \
  './transport_homogeneous_alternative \
  --network {slice} \
  --dt {dt} --seed 1 --time-bins 12 --time-bins-min 60 --time-bins-max 100000\
  --window-min $(echo {slice} | cut -d "-" -f 3 ) \
  --window-max $(echo {slice} | cut -d "-" -f 4 | cut -d "." -f 1) \
  --summary /dev/stdout' \
  :::: air_queue_dts.tsv ::: slice $prefix/slice-*.events \
  > $tmpdir/homogeneous-alt.jsonl;

mv $tmpdir/*.jsonl $prefix/
rm -rf $tmpir
