#!/bin/bash
#SBATCH --array=001-512:4
#SBATCH --time=1:00:00
#SBATCH --cpus-per-task 4
#SBATCH --mem=15GB
#SBATCH --job-name dists_ba_1024_8
#SBATCH -o /dev/null
#SBATCH -e /dev/null

set -ex


tmpdir=$(mktemp -d)
k=8;
perjob=4;
nodes=1024;
prefix=networks/random_large_networks/ba/$nodes/$k;

mkdir -p $tmpdir
mkdir -p $prefix

parallel -j $SLURM_CPUS_PER_TASK --header : --shuf -v \
  ./random_network --seed {seed} --nodes $nodes --max-time 2048 --mean-dt 1 \
  --static-model barabasi --temporal-model poisson \
  --average-degree $k '>' $tmpdir/s-{seed}-k-$k.events \
  ::: seed $(seq $SLURM_ARRAY_TASK_ID $(( $SLURM_ARRAY_TASK_ID + $perjob - 1 )))

parallel -j $SLURM_CPUS_PER_TASK --trim lr  --colsep ',' --header : --shuf --tmp=$tmpdir\
  ./network_degree_dist \
  --network  $tmpdir/s-{seed}-k-$k.events \
  --dt {dt} --seed 1 --sample-size 100000 \
  --degree-dists-file /dev/stdout \
  ::: seed $(seq $SLURM_ARRAY_TASK_ID $(( $SLURM_ARRAY_TASK_ID + $perjob - 1 ))) \
  :::: cor_len_queue_dts.tsv \
  > $tmpdir/s-$SLURM_ARRAY_TASK_ID-dist.jsonl;

mv $tmpdir/*.jsonl $prefix/
rm -rf $tmpir
