#!/bin/bash
#SBATCH --cpus-per-task 80
#SBATCH --ntasks-per-node 1
#SBATCH --nodes 1
#SBATCH --account=project_2003348
#SBATCH --time=3:00:00
#SBATCH --job-name dists_regular_4096_9
#SBATCH -o /dev/null
#SBATCH -e /dev/null

set -ex


tmpdir=$(mktemp -d --tmpdir=/dev/shm)
k=9;
nodes=4096;
prefix=networks/random_large_networks/regular/poisson/$nodes/$k;

mkdir -p $tmpdir
mkdir -p $prefix

parallel -j $SLURM_CPUS_PER_TASK --header : --shuf -v \
  python random_regular_network.py --seed {seed} --nodes $nodes \
  --average-degree $k '>' $tmpdir/s-{seed}.edges \
  ::: seed $(seq 1 64)

parallel -j $SLURM_CPUS_PER_TASK --header : --shuf -v \
  ./random_network --seed {seed} --nodes $nodes --max-time 2048 --mean-dt 1 \
  --static-model $tmpdir/s-{seed}.edges \
  --temporal-model poisson --average-degree $k \
  '>' $tmpdir/s-{seed}.events \
  ::: seed $(seq 1 64)

parallel -j $SLURM_CPUS_PER_TASK --trim lr  --colsep ',' --header : --shuf -v \
  ./network_degree_dist \
  --network  $tmpdir/s-{seed}.events \
  --dt {dt} --seed 1 --sample-size 10000 \
  --degree-dists-file /dev/stdout \
  ::: seed $(seq 1 64) \
  :::: regular_queue_dts.tsv \
  > $prefix/dist.jsonl;

rm -rf $tmpir
