#!/bin/bash
#SBATCH --ntasks-per-node 1
#SBATCH --nodes 1
#SBATCH --account=project_2003348
#SBATCH -o /dev/null
#SBATCH -e /dev/null

set -ex

tmpdir=$(mktemp -d --tmpdir=/dev/shm)
prefix=networks/random_large_networks/${static}_${dims}/poisson/${nodes}/$(( 2*dims ));
if [ $dims -eq 1 ];then
  queue_file=grid_1d_queue_dts.tsv
else
  queue_file=grid_2d_queue_dts.tsv
fi

mkdir -p $tmpdir
mkdir -p $prefix

python squarelike_grid_network.py --nodes $nodes --dimensions $dims \
  > $tmpdir/grid.edges

parallel -j $SLURM_CPUS_PER_TASK --header : --shuf -v \
  ./random_network --seed {seed} --nodes $nodes --max-time $max_time --mean-dt 1 \
  --static-model $tmpdir/grid.edges --average-degree $(( 2*dims )) \
  --temporal-model poisson \
  '>' $tmpdir/s-{seed}.events \
  ::: seed $(seq $SLURM_ARRAY_TASK_ID $(( $SLURM_ARRAY_TASK_ID + $perjob - 1 )))

parallel -j $SLURM_CPUS_PER_TASK --trim lr --colsep ',' --header : --shuf \
  ./random_large_homogeneous_alternative \
  --network $tmpdir/s-{seed}.events \
  --dt {dt} --seed 1 --time-bins 48 --time-bins-min 0.5 --time-bins-max $max_time \
  --window-min 0 --window-max $max_time --summary /dev/stdout \
  ::: seed $(seq $SLURM_ARRAY_TASK_ID $(( $SLURM_ARRAY_TASK_ID + $perjob - 1 ))) \
  :::: $queue_file \
  > $tmpdir/s-$SLURM_ARRAY_TASK_ID-homogeneous-alt.jsonl;

mv $tmpdir/*.jsonl $prefix/
rm -rf $tmpir
