import argparse
import collections

import networkx as nx

def sizes(n, dims):
  if dims < 1 or n < 1:
    raise "nodes and dimensions should be positive"

  if dims == 1:
    return [n]
  d = int(n**(1/dims))
  while n % d != 0:
    d -= 1
  return [d]+sizes(n//d, dims-1)

def ind_to_num(ind, dims):
  n = 0
  m = 1
  for i, d in zip(ind, dims):
    n += i*m
    m = d*m
  return n


if __name__ == "__main__":
  parser = argparse.ArgumentParser(
      description='generates a periodic square-like n-dimensional grid.')
  parser.add_argument('--nodes', required=True, type=int)
  parser.add_argument('--dimensions', required=True, type=int)

  args = parser.parse_args()

  dims = sizes(args.nodes, args.dimensions)
  net = nx.grid_graph(dims, periodic=True)

  for i, j in net.edges():
    if isinstance(i, collections.Iterable) and isinstance(j, collections.Iterable):
      print(ind_to_num(i[::-1], dims), ind_to_num(j[::-1], dims))
    else:
      print(i, j)
